#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Event handler to start or stop daemon "meimeikun-inotifyd"."""

import os
import stat

import ionas.request as request
import evh_config as config
import libevh.evhcmd as evhcmd
from fal_info import (FAL_SERVICECTRL_PATH, Results, ReqTypes)


EVENTINFO_NAME = "eventinfo"
VOLUME_NAME_PATH = "volume/name"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

VOLUME_CATEGORY = "volume"
MOUNT_ACTION = "mount"
UNMOUNT_ACTION = "unmount"

SERVICE_XPATH_TMPL = 'servicelist/service/name[text()="%s"]/..'
SERVICECTRL_REQ_XML_TMPL = """<servicectrl>
    <servicelist>
        <service><name>%s</name></service>
    </servicelist>
</servicectrl>
"""

INOTIFYD_SERVICE = "meimeikun-inotifyd"


class EvhMeiMeiKunINotifyD(evhcmd.EvhCmd):
    """Event handler to start or stop daemon "meimeikun-inotifyd"."""

    def __send_request(self, target, type, body=""):
        """Send request entry to "target".""" 

        entry = request.Entry(type, body)
        return self.send_entry(entry, target)

    def __service_control(self, name, type):
        """Control service."""

        self.__send_request(FAL_SERVICECTRL_PATH, type,
                        SERVICECTRL_REQ_XML_TMPL % (name,))

    def __handle_volume_event(self, eventinfo, entry):
        """Handle volume event."""

        action = eventinfo.get_attr_value(ACTION_ATTR)
        is_previous = eventinfo.is_set_to_true(PREVIOUS_ATTR)

        if (eventinfo.get_node(VOLUME_NAME_PATH).get_text() !=
                config.INTERNAL_VOL):

            # Non target events
            entry.set_result(Results.OK)
            return entry

        if action == MOUNT_ACTION:
            if not is_previous:
                self.__service_control(INOTIFYD_SERVICE, ReqTypes.START)

        elif action == UNMOUNT_ACTION:
            if is_previous:
                self.__service_control(INOTIFYD_SERVICE, ReqTypes.STOP)

        entry.set_result(Results.OK)
        return entry

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            eventinfo = entry.get().get_node(EVENTINFO_NAME)
            if eventinfo.get_attr_value(CATEGORY_ATTR) == VOLUME_CATEGORY:
                return self.__handle_volume_event(eventinfo, entry)

        except:
            # Non target events
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)


#
# Main
#

if __name__ == "__main__":
    EvhMeiMeiKunINotifyD(option=evhcmd.EvhCmdOption()).handle_request()
