# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from project.view import FuncView

import os
import datetime

import fields
from disk import Disk


class MeimeikunDiskFormatForm(forms.Form):

    def __init__(self, **kwargs):
        kwargs['prefix'] = kwargs.get('prefix', 'format')
        super(MeimeikunDiskFormatForm, self).__init__(**kwargs)

    year = fields.MeimeikunYearField()
    volumelabel = fields.MeimeikunVolumeLabelField()


class MeimeikunDiskArchiveScheduleForm(forms.Form):

    enabled = fields.MeimeikunArchiveScheduleEnabledField()
    dom = fields.MeimeikunArchiveScheduleDayOfMonthField(required=False)
    tod = fields.MeimeikunArchiveScheduleTimeOfDayField(required=False)

    def __init__(self, *args, **kwargs):
        if 'data' not in kwargs:
            kwargs['initial'] = kwargs.get('initial', self.__load_initial())
        super(MeimeikunDiskArchiveScheduleForm, self).__init__(*args, **kwargs)

    def __load_initial(self):
        try:
            from .._utils import load_config
            config = load_config()
            schedule = config['schedule_archive']
            return {
                'enabled': schedule['enabled'],
                'dom': schedule['day'],
                'tod': datetime.time(
                    schedule['time']['hour'],
                    schedule['time']['minute'],
                ),
            }
        except Exception as e:
            return {}

    def clean(self):
        cleaned_data = self.cleaned_data
        enabled = cleaned_data.get('enabled', False)
        dom = cleaned_data.get('dom', None)
        tod = cleaned_data.get('tod', None)
        if enabled:
            if dom is None:
                self._errors['dom'] = self.error_class([_(u"This field is required.")])
            if tod is None:
                self._errors['tod'] = self.error_class([_(u"This field is required.")])
        return cleaned_data


class MeimeikunDiskView(FuncView):

    views = ['save', 'format', 'archive', 'finalize', 'eject']

    def view(self, request):
        format_form = MeimeikunDiskFormatForm()
        archiveschedule_form = MeimeikunDiskArchiveScheduleForm()
        disk = Disk()

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'format_form': format_form,
            'archiveschedule_form': archiveschedule_form,
            'disk': disk,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_format(self, request):
        format_form = MeimeikunDiskFormatForm(data=request.POST)
        archiveschedule_form = MeimeikunDiskArchiveScheduleForm()
        form = format_form
        disk = Disk()

        with self.processing(constname='ERR_MEIMEIKUN_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            if form.is_valid():
                year = form.cleaned_data['year']
                volumelabel = form.cleaned_data['volumelabel']
                succeeded, message = disk.initialize(
                    year=year,
                    volumelabel=volumelabel,
                )
                proc.result(succeeded)
                self.set_message(message)
                disk = Disk()

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'form': form,
            'format_form': format_form,
            'archiveschedule_form': archiveschedule_form,
            'disk': disk,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_archive(self, request):
        format_form = MeimeikunDiskFormatForm()
        archiveschedule_form = MeimeikunDiskArchiveScheduleForm()
        disk = Disk()

        succeeded, message = disk.archive()
        self.message_is_error = succeeded is False
        self.set_message(message)
        disk = Disk()

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'format_form': format_form,
            'archiveschedule_form': archiveschedule_form,
            'disk': disk,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_finalize(self, request):
        format_form = MeimeikunDiskFormatForm()
        archiveschedule_form = MeimeikunDiskArchiveScheduleForm()
        disk = Disk()

        succeeded, message = disk.finalize()
        self.message_is_error = succeeded is False
        self.set_message(message)
        disk = Disk()

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'format_form': format_form,
            'archiveschedule_form': archiveschedule_form,
            'disk': disk,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_eject(self, request):
        format_form = MeimeikunDiskFormatForm()
        archiveschedule_form = MeimeikunDiskArchiveScheduleForm()
        disk = Disk()

        with self.processing(constname='ERR_MEIMEIKUN_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            succeeded = disk.eject()
            proc.result(succeeded)
            if succeeded:
                self.set_message('ディスクを取り出しました。')
            else:
                self.set_message('ディスクを取り出しできませんでした。')
            disk = Disk()

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'format_form': format_form,
            'archiveschedule_form': archiveschedule_form,
            'disk': disk,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_save(self, request):
        format_form = MeimeikunDiskFormatForm()
        archiveschedule_form = MeimeikunDiskArchiveScheduleForm(data=request.POST)
        form = archiveschedule_form
        disk = Disk()

        with self.processing(constname='ERR_MEIMEIKUN_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            if form.is_valid():
                enabled = form.cleaned_data['enabled']
                dom = form.cleaned_data['dom']
                tod = form.cleaned_data['tod']
                try:
                    from .._utils import update_config as save_config
                    succeeded, messages = save_config({
                        "schedule_archive": {
                            "enabled": enabled,
                            "day": dom,
                            "time": {
                                "hour": tod.hour,
                                "minute": tod.minute,
                            },
                        },
                    })
                    proc.result(succeeded)
                    self.set_message(message)
                except Exception as er:
                    pass

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'format_form': format_form,
            'archiveschedule_form': archiveschedule_form,
            'disk': disk,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }
